package com.bonus.sgzb.app.mapper;

import com.bonus.sgzb.app.domain.*;
import com.bonus.sgzb.base.api.domain.MaInputRecord;
import com.bonus.sgzb.base.api.domain.MaMachine;
import com.bonus.sgzb.base.api.domain.MaType;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;


/**
 * @author bns
 */
@Mapper
public interface PurchaseInputMapper {

    /**
     * 查询
     *
     * @param bean 查询条件
     * @return 结果
     */
    List<PurchaseCheckInfo> selectPutInListList(PurchaseCheckInfo bean);

    /**
     * 查询新购入库任务列表
     *
     * @param taskId 查询条件
     * @return 结果
     */
    String selectTypeNameByTaskId(@Param("taskId") Long taskId,@Param("type") String type);

    /**
     * 查询
     *
     * @param purchaseMacodeInfo 查询条件
     * @return 结果
     */
    List<PurchaseMacodeInfo> selectPutinDetails(PurchaseMacodeInfo purchaseMacodeInfo);

    /**
     * 修改
     *
     * @param maMachine
     * @return 结果
     */
    int updateMaMachine(MaMachine maMachine);

    /**
     * 查询
     *
     * @param typeId 查询条件
     * @return 结果
     */
    MaType selectTypeByTypeId(Long typeId);

    /**
     * 修改
     *
     * @param maType
     * @return 结果
     */
    int updateTypeByTypeId(MaType maType);

    /**
     * 查询
     * @param taskId 查询条件
     * @param typeId 查询条件
     * @return 结果
     */
    Integer selectInputNumByTypeId(@Param("taskId") Long taskId, @Param("typeId") Long typeId);

    /**
     * 修改
     *
     * @param purchaseCheckDetails
     * @return 结果
     */
    int updateByTaskIdTypeId(PurchaseCheckDetails purchaseCheckDetails);

    /**
     * 新增
     *
     * @param applyDetails
     * @return 结果
     */
    int insertInputApplyDetails(InputApplyDetails applyDetails);

    /**
     * 修改
     *
     * @param purchaseMacodeInfo
     * @return 结果
     */
    int updateMacodeByType(PurchaseMacodeInfo purchaseMacodeInfo);

    /**
     * 新增
     *
     * @param maInputRecord
     * @return 结果
     */
    int insertMaInputRecord(MaInputRecord maInputRecord);

    /**
     * 查询
     *
     * @param taskId 查询条件
     * @return 结果
     */
    Integer selectMacodeInfoStatusByTaskId(Long taskId);

    /**
     * 修改
     *
     * @param task
     * @return 结果
     */
    int updateTmTask(TmTask task);
    /**
     * 判断是否全部已操作（不通过或通过）
     * @param taskId
     * @return int
     */
    int isOperateAll(Long taskId);
    /**
     * 是否为全部不通过
     * @param taskId
     * @return
     */
    int selectPurchaseCheckDetailsStatus(Long taskId);

    /**
     * 删除ma_machine表
     * @param maId
     * @return 结果
     */
    int deleteMaMachineInfoByMaId(Long maId);

    /**
     * 删除ma_machine_label表
     * @param maId
     * @return int
     */
    int deleteMaMachineLabelByMaId(Long maId);

    /**
     * 删除ma_label_bind表
     * @param maId
     * @return int
     */
    int deleteMaLabelBindByMaId(Long maId);

    /**
     * 删除purchase_macode_info表
     * @param taskId
     * @param typeId
     * @param maCode
     * @return
     */
    int updatetePurchaseMaCodeInfoByMaCodeAndTaskIdAndTypeId(@Param("taskId") Long taskId,@Param("typeId") Long typeId,@Param("maCode") String maCode);

    /**
     * 修改purchase_check_details表
     * @param typeId
     * @param taskId
     * @return int
     */
    int updatePurchaseCheckDetailsByTaskId(@Param("taskId") Long taskId,@Param("typeId") Long typeId);

    /**
     * 查询组织是否开启审核
     * @param companyId
     * @return int
     */
    int selectExamineType(Long companyId);

    /**
     * 查询
     * @param bean
     * @return
     */
    List<PurchaseCheckInfo> selectPutInListExamine(PurchaseCheckInfo bean);
}